@@ -42,6 +42,8 @@ int MGL_EXPORT mgl_fltk_thr();
 /// Wrapper class for windows displaying graphics
 class MGL_EXPORT mglFLTK : public mglWnd
 {
+	mglFLTK(const mglFLTK &t) {}	// copying is not allowed
+	const mglFLTK &operator=(const mglFLTK &t)	{	return t;	}
 public:
 	mglFLTK(const char *title="MathGL") : mglWnd()
 	{	gr = mgl_create_graph_fltk(0,title,0,0);	}
@@ -52,6 +54,7 @@ public:
 	mglFLTK(mglDraw *draw, const char *title="MathGL") : mglWnd()
 	{	gr = mgl_create_graph_fltk(draw?mgl_draw_class:0,title,draw,mgl_reload_class);
 		mgl_set_click_func(gr, mgl_click_class);	}
+    virtual ~mglFLTK() {}
 	int Run()	{	return mgl_fltk_run();	}	///< Run main loop for event handling
 	int RunThr()	{	return mgl_fltk_thr();	}	///< Run main loop for event handling in separate thread
 };
@@ -61,8 +64,8 @@ public:
 #endif
 
 #include <FL/Fl.H>
-#include <Fl/Fl_Window.H>
-#include <Fl/Fl_Scroll.H>
+#include <FL/Fl_Window.H>
+#include <FL/Fl_Scroll.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Counter.H>
 #include <FL/Fl_Menu_Bar.H>
@@ -76,7 +79,7 @@ public:
 	Fl_Valuator	*phi_val;	///< pointer to external phi-angle validator
 
 	Fl_MathGL(int x, int y, int w, int h, const char *label=0);
-	~Fl_MathGL();
+	virtual ~Fl_MathGL();
 
 	/// Update (redraw) plot
 	virtual void update();
@@ -108,6 +111,7 @@ public:
 	{	popup = pmenu;	wpar = wdg;	vpar = v;	}
 	inline void zoom_region(mreal xx1,mreal xx2,mreal yy1, mreal yy2)
 	{	x1=xx1;	y1=yy1;	x2=xx2;	y2=yy2;	}
+	void stop(bool stop=true);
 
 protected:
 	mglCanvas *gr;		///< pointer to grapher
@@ -118,18 +122,18 @@ protected:
 
 	const Fl_Menu_Item *popup;	///< pointer to popup menu items
 	Fl_Widget *wpar;			///< widget for popup menu
-	void *vpar;				///< parameter for popup menu
-	mreal tet,phi;			///< rotation angles
+	void *vpar;					///< parameter for popup menu
+	mreal tet,phi;				///< rotation angles
 	bool rotate;				///< flag for handle mouse
-	bool zoom;				///< flag for zoom by mouse
+	bool zoom;					///< flag for zoom by mouse
 	bool wire;
-	mreal x1,x2,y1,y2;		///< zoom region
-	int flag;				///< bitwise flag for general state (1-Alpha, 2-Light)
+	mreal x1,x2,y1,y2;			///< zoom region
+	int flag;					///< bitwise flag for general state (1-Alpha, 2-Light)
 	int x0,y0,xe,ye;			///< mouse position
 	char pos[128];
 
 	virtual void draw();		///< quick drawing function
-	int handle(int code);	///< handle mouse events
+	int handle(int code);		///< handle mouse events
 	void resize(int x, int y, int w, int h);	///< resize control
 };
 //-----------------------------------------------------------------------------
@@ -159,7 +163,7 @@ public:
 	{	mgl_set_size(FMGL->get_graph(),scroll->w(),scroll->h());	FMGL->size(scroll->w(),scroll->h());	update();	}
 
 	Fl_MGLView(int x, int y, int w, int h, const char *label=0);
-	~Fl_MGLView();
+	virtual ~Fl_MGLView();
 	void update();			///< Update picture by calling user drawing function
 protected:
 	Fl_Button *alpha_bt, *light_bt, *rotate_bt, *anim_bt, *zoom_bt, *grid_bt;
