@@ -39,9 +39,10 @@ void MGL_EXPORT mgl_mpi_recv_(uintptr_t *gr, int *id);
 /// Wrapper class for all graphics
 class MGL_EXPORT mglGraphMPI:public mglGraph
 {
+	mglGraphMPI(const mglGraphMPI &t) {}	// copying is not allowed
+	const mglGraphMPI &operator=(const mglGraphMPI &t)	{	return t;	}
 public:
 	inline mglGraphMPI(int kind=0, int width=600, int height=400):mglGraph(kind,width,height){}
-	inline mglGraphMPI(const mglGraph &graph):mglGraph(graph){}
 	inline mglGraphMPI(HMGL graph):mglGraph(graph){}
 	virtual ~mglGraphMPI(){}
 
