@@ -26,55 +26,105 @@ extern "C" {
 #endif
 //-----------------------------------------------------------------------------
 /// Draw isosurface for 3d data specified parametrically
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.*/
 void MGL_EXPORT mgl_surf3_xyz_val(HMGL graph, double Val, HCDT x, HCDT y, HCDT z, HCDT a, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3_xyz_val_(uintptr_t *graph, mreal *Val, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *stl, const char *opt,int,int);
 /// Draw isosurface for 3d data
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.*/
 void MGL_EXPORT mgl_surf3_val(HMGL graph, double Val, HCDT a, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3_val_(uintptr_t *graph, mreal *Val, uintptr_t *a, const char *stl, const char *opt,int,int);
 /// Draw isosurfaces for 3d data specified parametrically
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
 void MGL_EXPORT mgl_surf3_xyz(HMGL graph, HCDT x, HCDT y, HCDT z, HCDT a, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *stl, const char *opt,int,int);
 /// Draw isosurfaces for 3d data
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
 void MGL_EXPORT mgl_surf3(HMGL graph, HCDT a, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3_(uintptr_t *graph, uintptr_t *a, const char *stl, const char *opt,int,int);
 
 /// Draw isosurface for 3d data specified parametrically with alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots. */
 void MGL_EXPORT mgl_surf3a_xyz_val(HMGL graph, double Val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3a_xyz_val_(uintptr_t *graph, mreal *Val, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 /// Draw isosurface for 3d data with alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots. */
 void MGL_EXPORT mgl_surf3a_val(HMGL graph, double Val, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3a_val_(uintptr_t *graph, mreal *Val, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 /// Draw isosurfaces for 3d data specified parametrically with alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
 void MGL_EXPORT mgl_surf3a_xyz(HMGL graph, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3a_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 /// Draw isosurfaces for 3d data with alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
 void MGL_EXPORT mgl_surf3a(HMGL graph, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3a_(uintptr_t *graph, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 
 /// Draw isosurface for 3d data specified parametrically with color proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots. */
 void MGL_EXPORT mgl_surf3c_xyz_val(HMGL graph, double Val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3c_xyz_val_(uintptr_t *graph, mreal *Val, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 /// Draw isosurface for 3d data with color proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots. */
 void MGL_EXPORT mgl_surf3c_val(HMGL graph, double Val, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3c_val_(uintptr_t *graph, mreal *Val, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 /// Draw isosurfaces for 3d data specified parametrically with color proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
 void MGL_EXPORT mgl_surf3c_xyz(HMGL graph, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3c_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 /// Draw isosurfaces for 3d data with color proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
 void MGL_EXPORT mgl_surf3c(HMGL graph, HCDT a, HCDT b, const char *stl, const char *opt);
 void MGL_EXPORT mgl_surf3c_(uintptr_t *graph, uintptr_t *a, uintptr_t *b, const char *stl, const char *opt,int,int);
 
+/// Draw isosurface for 3d data specified parametrically with color proportional to c and alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots. */
+void MGL_EXPORT mgl_surf3ca_xyz_val(HMGL graph, double Val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT c, HCDT b, const char *stl, const char *opt);
+void MGL_EXPORT mgl_surf3ca_xyz_val_(uintptr_t *graph, mreal *Val, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *stl, const char *opt,int,int);
+/// Draw isosurface for 3d data with color proportional to c and alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots. */
+void MGL_EXPORT mgl_surf3ca_val(HMGL graph, double Val, HCDT a, HCDT c, HCDT b, const char *stl, const char *opt);
+void MGL_EXPORT mgl_surf3ca_val_(uintptr_t *graph, mreal *Val, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *stl, const char *opt,int,int);
+/// Draw isosurfaces for 3d data specified parametrically with color proportional to c and alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
+void MGL_EXPORT mgl_surf3ca_xyz(HMGL graph, HCDT x, HCDT y, HCDT z, HCDT a, HCDT c, HCDT b, const char *stl, const char *opt);
+void MGL_EXPORT mgl_surf3ca_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *stl, const char *opt,int,int);
+/// Draw isosurfaces for 3d data with color proportional to c and alpha proportional to b
+/** Style ‘#’ draw wired plot. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3). */
+void MGL_EXPORT mgl_surf3ca(HMGL graph, HCDT a, HCDT c, HCDT b, const char *stl, const char *opt);
+void MGL_EXPORT mgl_surf3ca_(uintptr_t *graph, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *stl, const char *opt,int,int);
+
 /// Draw a semi-transparent cloud for 3d data specified parametrically
+/** Style ‘.’ produce plot by dots. Style ‘i’ use inverted values for transparency. */
 void MGL_EXPORT mgl_cloud_xyz(HMGL graph, HCDT x, HCDT y, HCDT z, HCDT a, const char *stl, const char *opt);
 void MGL_EXPORT mgl_cloud_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *stl, const char *opt,int,int);
 /// Draw a semi-transparent cloud for 3d data
+/** Style ‘.’ produce plot by dots. Style ‘i’ use inverted values for transparency. */
 void MGL_EXPORT mgl_cloud(HMGL graph, HCDT a, const char *stl, const char *opt);
 void MGL_EXPORT mgl_cloud_(uintptr_t *graph, uintptr_t *a, const char *stl, const char *opt,int,int);
 
 /// Draw isosurface for 3d beam in curvilinear coordinates
+/** Style ‘#’ draw grid lines. Style ‘.’ produce plot by dots.
+ *  Variable \a flag is bitwise:
+ * ‘0x1’ - draw in accompanied (not laboratory) coordinates;
+ * ‘0x2’ - draw projection to \rho-z plane;
+ * ‘0x4’ - draw normalized in each slice field.*/
 void MGL_EXPORT mgl_beam_val(HMGL graph, double Val, HCDT tr, HCDT g1, HCDT g2, HCDT a, double r, const char *stl, int norm);
 void MGL_EXPORT mgl_beam_val_(uintptr_t *gr, mreal *val, uintptr_t *tr, uintptr_t *g1, uintptr_t *g2, uintptr_t *a, mreal *r, const char *sch, int *norm,int l);
 /// Draw several isosurfaces for 3d beam in curvilinear coordinates
+/** Style ‘#’ draw grid lines. Style ‘.’ produce plot by dots.
+ * Option "value" set the number of isosurfaces (default is 3).
+ *  Variable \a flag is bitwise:
+ * ‘0x1’ - draw in accompanied (not laboratory) coordinates;
+ * ‘0x2’ - draw projection to \rho-z plane;
+ * ‘0x4’ - draw normalized in each slice field.*/
 void MGL_EXPORT mgl_beam(HMGL graph, HCDT tr, HCDT g1, HCDT g2, HCDT a, double r, const char *stl, int norm, int num);
 void MGL_EXPORT mgl_beam_(uintptr_t *gr, uintptr_t *tr, uintptr_t *g1, uintptr_t *g2, uintptr_t *a, mreal *r, const char *sch, int *norm, int *num,int l);
 //-----------------------------------------------------------------------------
