@@ -57,6 +57,7 @@ EQ_LN,		// logarithm of x, ln(x)
 EQ_LG,		// decimal logarithm of x, lg(x) = ln(x)/ln(10)
 EQ_ABS,		// absolute value
 EQ_ARG,		// argument (or phase) of complex number
+EQ_CONJ,	// complex conjugate
 EQ_LAST		// id of last entry
 };
 //-----------------------------------------------------------------------------
@@ -157,6 +158,7 @@ mglFormulaC::mglFormulaC(const char *string)
 		else if(!strcmp(name,"ln")) Kod=EQ_LN;
 		else if(!strcmp(name,"abs")) Kod=EQ_ABS;
 		else if(!strcmp(name,"arg")) Kod=EQ_ARG;
+		else if(!strcmp(name,"conj")) Kod=EQ_CONJ;
 		else {	delete []str;	return;	}	// unknown function
 		n=mglFindInText(str,",");
 		if(n>=0)
@@ -220,6 +222,7 @@ dual MGL_NO_EXPORT ic = dual(0,1);
 dual MGL_LOCAL_CONST asinhc(dual x)	{	return log(x+sqrt(x*x+mreal(1)));	}
 dual MGL_LOCAL_CONST acoshc(dual x)	{	return log(x+sqrt(x*x-mreal(1)));	}
 dual MGL_LOCAL_CONST atanhc(dual x)	{	return log((mreal(1)+x)/(mreal(1)-x))/mreal(2);	}
+dual MGL_LOCAL_CONST conjc(dual x)	{	return dual(real(x),-imag(x));	}
 dual MGL_LOCAL_CONST sinc(dual x)	{	return sin(x);	}
 dual MGL_LOCAL_CONST cosc(dual x)	{	return cos(x);	}
 dual MGL_LOCAL_CONST tanc(dual x)	{	return tan(x);	}
@@ -243,7 +246,7 @@ dual mglFormulaC::CalcIn(const dual *a1) const
 {
 	func_2 f2[EQ_SIN-EQ_ADD] = {addc,subc,mulc,divc,ipwc,powc,llgc};
 	func_1 f1[EQ_LAST-EQ_SIN] = {sinc,cosc,tanc,asinc,acosc,atanc,sinhc,coshc,tanhc,
-					asinhc,acoshc,atanhc,sqrtc,expc,expi,logc,lgc,absc};
+					asinhc,acoshc,atanhc,sqrtc,expc,expi,logc,lgc,absc,argc,conjc};
 //	if(Error)	return 0;
 	if(Kod==EQ_A)	return a1[(int)Res.real()];
 	if(Kod==EQ_RND)	return mgl_rnd();
@@ -267,15 +270,18 @@ dual mglFormulaC::CalcIn(const dual *a1) const
 mdual MGL_EXPORT_CONST mgl_ipowc(dual x,int n)
 {
 	dual t;
-	if(n==2)	{	t = x*x;	return t.real()+t.imag()*_Complex_I;	}
-	if(n==1)	return x.real()+x.imag()*_Complex_I;
-	if(n<0)		{	t = mreal(1)/mgl_ipowc(x,-n);	return t.real()+t.imag()*_Complex_I;	}
-	if(n==0)	return mreal(1);
-	t = mgl_ipowc(x,n/2);	t = t*t;
-	if(n%2==1)	t *= x;
-	return t.real()+t.imag()*_Complex_I;
+	if(n==2)	t = x*x;
+	else if(n==1)	t = x;
+	else if(n<0)	t = mreal(1)/mgl_ipowc(x,-n);
+	else if(n==0)	t = mreal(1);
+	else
+	{
+		t = mgl_ipowc(x,n/2);	t = t*t;
+		if(n%2==1)	t *= x;
+	}
+	return t.real()+t.imag()*mgl_I;
 }
-mdual MGL_EXPORT_PURE mgl_ipowc_(dual *x,int *n)	{	return mgl_ipowc(*x,*n);	}
+mdual MGL_EXPORT mgl_ipowc_(dual *x,int *n)	{	return mgl_ipowc(*x,*n);	}
 //-----------------------------------------------------------------------------
 HAEX MGL_EXPORT mgl_create_cexpr(const char *expr)	{	return new mglFormulaC(expr);	}
 uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int l)
@@ -284,10 +290,10 @@ uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int l)
 	delete []s;	return res;	}
 void MGL_EXPORT mgl_delete_cexpr(HAEX ex)	{	if(ex)	delete ex;	}
 void MGL_EXPORT mgl_delete_cexpr_(uintptr_t *ex)	{	mgl_delete_cexpr((HAEX)ex);	}
-mdual MGL_EXPORT_PURE mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z)
-{	dual r = ex->Calc(x,y,z);	return r.real()+r.imag()*_Complex_I;	}
+mdual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z)
+{	dual r = ex->Calc(x,y,z);	return r.real()+r.imag()*mgl_I;	}
 mdual MGL_EXPORT mgl_cexpr_eval_(uintptr_t *ex, dual *x, dual *y, dual *z)
 {	return mgl_cexpr_eval((HAEX) ex, *x,*y,*z);		}
 mdual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *var)
-{	dual r = ex->Calc(var);	return r.real()+r.imag()*_Complex_I;	}
+{	dual r = ex->Calc(var);	return r.real()+r.imag()*mgl_I;	}
 //-----------------------------------------------------------------------------
