@@ -48,7 +48,7 @@ long mglParser::IsFunc(const std::wstring &name, int *na)
 	{
 		const mglFunc &f = func[i];
 		if(f.func==name)
-		{	if(na)	*na=f.narg;	return f.pos;	}
+		{	if(na)	*na=f.narg;	return f.pos+1;	}
 	}
 	return 0;
 }
@@ -80,9 +80,10 @@ int MGL_LOCAL_PURE mgl_cmd_cmp(const void *a, const void *b)
 	return strcmp(aa->name, bb->name);
 }
 //-----------------------------------------------------------------------------
-bool mgl_check_for_name(const std::wstring &s)
+bool mglParser::CheckForName(const std::wstring &s)
 {
-	return !isalpha(s[0])||s.find_first_of(L".:()")!=std::wstring::npos;
+	return !isalpha(s[0]) || s.find_first_of(L"!@#$%%^&*()-+|,.<>:")!=std::wstring::npos || s==L"rnd" || FindNum(s.c_str());
+//	return !isalpha(s[0])||s.find_first_of(L".:()")!=std::wstring::npos;
 }
 //-----------------------------------------------------------------------------
 const mglCommand *mglParser::FindCommand(const char *com) const
@@ -120,12 +121,12 @@ int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const s
 	}
 	const mglCommand *rts=FindCommand(com);
 	if(!rts || rts->type==6)	return 2;
-	if(rts->type == 4)
+/*	if(rts->type == 4)
 	{
-		if(n<1 || mgl_check_for_name(var))	return 2;
+		if(n<1 || CheckForName(var))	return 2;
 		a[0].type = 0;	a[0].d = AddVar(var.c_str());
 		a[0].w = var;	k[0] = 'd';
-	}
+	}*/
 	char *o=0;
 	if(opt && *opt)	// TODO: parse arguments of options
 	{
@@ -152,6 +153,12 @@ mglParser::mglParser(bool setsize)
 	AllowSetSize=setsize;	AllowFileIO=true;	AllowDllCall=true;
 	Once = true;
 	fval = new mglData[40];
+	mglNum *v;
+	v = new mglNum(0);	v->s = L"off";	NumList.push_back(v);
+	v = new mglNum(1);	v->s = L"on";	NumList.push_back(v);
+	v = new mglNum(NAN);	v->s = L"nan";	NumList.push_back(v);
+	v = new mglNum(M_PI);	v->s = L"pi";	NumList.push_back(v);
+	v = new mglNum(INFINITY);	v->s = L"inf";	NumList.push_back(v);
 #if MGL_HAVE_LTDL
 	lt_dlinit();
 #endif
@@ -173,6 +180,12 @@ void mglParser::DeleteAll()
 	for(size_t i=0;i<NumList.size();i++)
 		if(NumList[i])	delete NumList[i];
 	NumList.clear();
+	mglNum *v;
+	v = new mglNum(0);	v->s = L"off";	NumList.push_back(v);
+	v = new mglNum(1);	v->s = L"on";	NumList.push_back(v);
+	v = new mglNum(NAN);	v->s = L"nan";	NumList.push_back(v);
+	v = new mglNum(M_PI);	v->s = L"pi";	NumList.push_back(v);
+	v = new mglNum(INFINITY);	v->s = L"inf";	NumList.push_back(v);
 	if(Cmd && Cmd!=mgls_base_cmd)
 	{	delete []Cmd;	Cmd = mgls_base_cmd;	}
 #if MGL_HAVE_LTDL
@@ -194,9 +207,9 @@ int mglParser::Parse(mglGraph *gr, const char *str, long pos)
 	return r;
 }
 //-----------------------------------------------------------------------------
-mglData *mglParser::AddVar(const char *str)
+mglDataA *mglParser::AddVar(const char *str)
 {
-	mglData *v=0;
+	mglDataA *v=0;
 	MGL_TO_WCS(str,v = AddVar(wcs));
 	return v;
 }
@@ -230,22 +243,19 @@ void mglParser::AddParam(int n, const wchar_t *str)
 //-----------------------------------------------------------------------------
 mglDataA *mglParser::FindVar(const wchar_t *name)
 {
+	if(name[0]=='!')	name = name+1;	// ignore complex prefix
  	for(size_t i=0;i<DataList.size();i++)
  		if(DataList[i] && DataList[i]->s==name)	return DataList[i];
 	return 0;
 }
 //-----------------------------------------------------------------------------
-mglData *mglParser::AddVar(const wchar_t *name)
+mglDataA *mglParser::AddVar(const wchar_t *name)
 {	// TODO add list of forbidden names (like function names)
- 	for(size_t i=0;i<DataList.size();i++)
- 		if(DataList[i] && DataList[i]->s==name)
-		{
-			delete (DataList[i]);
-			mglData *d = new mglData;	d->s = name;
-			DataList[i] = d;	return d;
-		}
-	mglData *d = new mglData;	d->s = name;
-	DataList.push_back(d);
+	mglDataA *d=FindVar(name);
+	if(name[0]=='!' && dynamic_cast<mglDataC*>(d)==0)
+	{	d = new mglDataC;	d->s=(name+1);	DataList.push_back(d);	}
+	else if(!d)
+	{	d = new mglData;	d->s = name;	DataList.push_back(d);	}
 	return d;
 }
 //-----------------------------------------------------------------------------
@@ -259,12 +269,7 @@ mglNum *mglParser::FindNum(const wchar_t *name)
 mglNum *mglParser::AddNum(const wchar_t *name)
 {
 	mglNum *v = FindNum(name);
-	if(!v)
-	{
-		mglNum *n=new mglNum;	n->s = name;
-		NumList.push_back(n);
-		v = NumList[NumList.size()-1];
-	}
+	if(!v)	{	v=new mglNum;	v->s = name;	NumList.push_back(v);	}
 	return v;
 }
 //-----------------------------------------------------------------------------
@@ -285,8 +290,10 @@ int MGL_LOCAL_PURE mglFindArg(const std::wstring &str)
 	return 0;
 }
 //-----------------------------------------------------------------------------
+HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring string, mglParser *arg, const std::vector<mglDataA*> &head);
+HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring string, mglParser *arg, const std::vector<mglDataA*> &head);
+//-----------------------------------------------------------------------------
 // convert substrings to arguments
-mglData MGL_NO_EXPORT mglFormulaCalc(std::wstring string, mglParser *arg, const std::vector<mglDataA*> &head);
 void mglParser::FillArg(mglGraph *gr, int k, std::wstring *arg, mglArg *a)
 {
 	register long n;
@@ -312,8 +319,20 @@ void mglParser::FillArg(mglGraph *gr, int k, std::wstring *arg, mglArg *a)
 					for(;i<ll && w[i]!='\'';i++);
 					if(i>i1)
 					{
-						mglData d = mglFormulaCalc(w.substr(i1,i-i1-(w[i]=='\''?1:0)), this, DataList);
-						mglprintf(buf,32,L"%g",d.a[0]);	a[n-1].w += buf;
+						if(w[i1]=='!')
+						{
+							HADT d = mglFormulaCalcC(w.substr(i1+1,i-i1-(w[i]=='\''?1:0)), this, DataList);
+							mreal di = imag(d->a[0]), dr = real(d->a[0]);
+							if(di>0)	mglprintf(buf,32,L"%g+%gi",dr,di);
+							else if(di<0)	mglprintf(buf,32,L"%g-%gi",dr,-di);	// TODO use \u2212 ???
+							else	mglprintf(buf,32,L"%g",dr);
+							a[n-1].w += buf;	delete d;
+						}
+						else
+						{
+							HMDT d = mglFormulaCalc(w.substr(i1,i-i1-(w[i]=='\''?1:0)), this, DataList);
+							mglprintf(buf,32,L"%g",d->a[0]);	a[n-1].w += buf;	delete d;
+						}
 					}
 				}
 				else	a[n-1].w += w[i];
@@ -331,18 +350,41 @@ void mglParser::FillArg(mglGraph *gr, int k, std::wstring *arg, mglArg *a)
 		else if((v = FindVar(arg[n].c_str()))!=0)	// try to find normal variables (for data creation)
 		{	a[n-1].type=0;	a[n-1].d=v;	a[n-1].w=v->s;	}
 		else if((f = FindNum(arg[n].c_str()))!=0)	// try to find normal number (for data creation)
-		{	a[n-1].type=2;	a[n-1].d=0;	a[n-1].v=f->d;	a[n-1].w = f->s;	}
-		else
+		{	a[n-1].type=2;	a[n-1].d=0;	a[n-1].v=f->d;	a[n-1].c=f->c;	a[n-1].w = f->s;	}
+		else if(arg[n][0]=='!')	// complex array is asked
 		{	// parse all numbers and formulas by unified way
-			mglData d = mglFormulaCalc(arg[n], this, DataList);
-			if(d.nx*d.ny*d.nz==1)
-			{	a[n-1].type = 2;	a[n-1].v = d.a[0];	}
+			HADT d = mglFormulaCalcC(arg[n].substr(1), this, DataList);
+			if(d->GetNN()==1)
+			{
+				if(CheckForName(arg[n].substr(1)))
+				{	a[n-1].type = 2;	a[n-1].v = d->v(0);	a[n-1].c = d->a[0];	}
+				else
+				{	a[n-1].type = 0;	a[n-1].d = AddVar(arg[n].c_str());	}
+				delete d;
+			}
 			else
 			{
-				mglData *u=new mglData;	u->Set(d);
 				a[n-1].w = L"/*"+arg[n]+L"*/";
-				u->temp=true;	DataList.push_back(u);
-				a[n-1].type = 0;	a[n-1].d = u;
+				d->temp=true;	DataList.push_back(d);
+				a[n-1].type = 0;	a[n-1].d = d;
+			}
+		}
+		else
+		{	// parse all numbers and formulas by unified way
+			HMDT d = mglFormulaCalc(arg[n], this, DataList);
+			if(d->GetNN()==1)
+			{
+				if(CheckForName(arg[n]))
+				{	a[n-1].type = 2;	a[n-1].c = a[n-1].v = d->v(0);	}
+				else
+				{	a[n-1].type = 0;	a[n-1].d = AddVar(arg[n].c_str());	}
+				delete d;
+			}
+			else
+			{
+				a[n-1].w = L"/*"+arg[n]+L"*/";
+				d->temp=true;	DataList.push_back(d);
+				a[n-1].type = 0;	a[n-1].d = d;
 			}
 		}
 	}
@@ -356,7 +398,7 @@ int mglParser::PreExec(mglGraph *, long k, std::wstring *arg, mglArg *a)
 	{	DeleteVar(arg[1].c_str());	n=1;	}
 	else if(!arg[0].compare(L"list"))	// parse command "list"
 	{
-		if(k<3 || mgl_check_for_name(arg[1]))	return 2;
+		if(k<3 || CheckForName(arg[1]))	return 2;
 		long nx=0, ny=1,j=0,i,t=0;
 		for(i=2;i<k;i++)
 		{
@@ -374,54 +416,51 @@ int mglParser::PreExec(mglGraph *, long k, std::wstring *arg, mglArg *a)
 			}
 			if(ch=='|' && t==1)		{	nx = j>nx ? j:nx;	j=0;	ny++;	}
 		}
-		mglData *v = AddVar(arg[1].c_str());
-		if(t==1)	nx = j>nx ? j:nx;
-		if(t==1)	// list of numeric values
+		mglDataA *vv = AddVar(arg[1].c_str());
+		mglData *v = dynamic_cast<mglData*>(vv);
+		mglDataC *vc = dynamic_cast<mglDataC*>(vv);
+		if(v)
 		{
-			v->Create(nx,ny);
-			j=t=0;
-			for(i=2;i<k;i++)
+			if(t==1)	nx = j>nx ? j:nx;
+			if(t==1)	// list of numeric values
 			{
-				if(arg[i][0]=='|')	{	t++;	j=0;	}
-				else
-				{	v->a[j+nx*t] = a[i-1].v;	j++;	}
-			}
-		}
-		if(t==2)	// list of data
-		{
-			mglData *b = dynamic_cast<mglData *>(a[1].d);
-			if(!b)	return 1;
-			long nn = 0;
-			if(b->nz>1)	return 2;
-			if(b->ny>1)
-			{
-				v->Create(b->nx, b->ny, nx);
-				nn = b->nx*b->ny;
-				for(i=2,j=0;i<k;i++)
+				v->Create(nx,ny);
+				j=t=0;
+				for(i=2;i<k;i++)
 				{
-					if(nn!=a[i-1].d->GetNx()*a[i-1].d->GetNy())	continue;
-					b = dynamic_cast<mglData *>(a[i-1].d);
-					if(b)	memcpy(v->a+j*nn,b->a,nn*(b->nz)*sizeof(mreal));
-					else	for(long ii=0;ii<a[i-1].d->GetNN();ii++)	v->a[ii+j*nn] = a[i-1].d->vthr(ii);
-					j+=a[i-1].d->GetNz();
+					if(arg[i][0]=='|')	{	t++;	j=0;	}
+					else
+					{	v->a[j+nx*t] = a[i-1].v;	j++;	}
 				}
 			}
-			else
+			if(t==2)	// list of data
 			{
-				v->Create(b->nx, nx);
-				nn = b->nx;
-				for(i=2,j=0;i<k;i++)
+				v->Set(a[1].d);
+				for(long i=2;i<k;i++)	v->Join(*(a[i].d));
+			}
+			n=1;
+		}
+		if(vc)
+		{
+			if(t==1)	nx = j>nx ? j:nx;
+			if(t==1)	// list of numeric values
+			{
+				vc->Create(nx,ny);
+				j=t=0;
+				for(i=2;i<k;i++)
 				{
-					if(nn!=a[i-1].d->GetNx())	continue;
-					b = dynamic_cast<mglData *>(a[i-1].d);
-					long nny = a[i-1].d->GetNy();
-					if(b)	memcpy(v->a+j*nn,b->a,nn*nny*sizeof(mreal));
-					else	for(long ii=0;ii<nn*nny;ii++)	v->a[ii+j*nn] = a[i-1].d->vthr(ii);
-					j+=nny;
+					if(arg[i][0]=='|')	{	t++;	j=0;	}
+					else
+					{	vc->a[j+nx*t] = a[i-1].c;	j++;	}
 				}
 			}
+			if(t==2)	// list of data
+			{
+				vc->Set(a[1].d);
+				for(long i=2;i<k;i++)	vc->Join(*(a[i].d));
+			}
+			n=1;
 		}
-		n=1;
 	}
 	return n;
 }
@@ -433,11 +472,12 @@ void mglParser::PutArg(std::wstring &str, bool def)
 	{
 		wchar_t ch = str[pos+1];
 		if(ch>='0' && ch<='9')	str.replace(pos,2,par[ch-'0']);
-		if(ch>='a' && ch<='z')	str.replace(pos,2,par[ch-'a'+10]);
-		if(ch=='$')	str.replace(pos,2,L"\xffff");
+		else if(ch>='a' && ch<='z')	str.replace(pos,2,par[ch-'a'+10]);
+		else if(ch=='$')	str.replace(pos,2,L"\uffff");
+		else str.replace(pos,1,L"\uffff");
 		pos = str.find('$',def?10:0);
 	}
-	while((pos = str.find(L'\xffff'))<str.length())	str[pos]='$';
+	while((pos = str.find(L'\uffff'))<str.length())	str[pos]='$';
 }
 //-----------------------------------------------------------------------------
 std::wstring mgl_trim_ws(const std::wstring &str)
@@ -469,7 +509,8 @@ int mglParser::ParseDef(std::wstring &str)
 			if(s[0]=='$' && nn>=0 && nn<='z'-'a'+10)
 			{
 				res = 0;
-				d = mglFormulaCalc(mgl_trim_ws(s.substr(2)), this, DataList).a[0];
+				HMDT dd = mglFormulaCalc(mgl_trim_ws(s.substr(2)), this, DataList);
+				d = dd->a[0];	delete dd;
 				char buf[32];	snprintf(buf,32,"%g",d);
 				buf[31] = 0;	AddParam(nn, buf);
 			}
@@ -481,7 +522,8 @@ int mglParser::ParseDef(std::wstring &str)
 			if(s[0]=='$' && nn>=0 && nn<='z'-'a'+10)
 			{
 				res = 0;
-				d=mglFormulaCalc(mgl_trim_ws(s.substr(2)), this, DataList).a[0];
+				HMDT dd = mglFormulaCalc(mgl_trim_ws(s.substr(2)), this, DataList);
+				d=dd->a[0];	delete dd;
 				wchar_t buf[2]={0,0};	buf[0] = wchar_t(d);	AddParam(nn, buf);
 			}
 			return res+1;
@@ -583,7 +625,7 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 		if(!skip() && !arg[0].compare(L"stop"))
 		{	Stop = true;	delete []a;	return 0;	}
 		if(!arg[0].compare(L"func"))
-		{	delete []a;	return 0;	}
+		{	Stop = true;	delete []a;	return 0;	}
 		n = FlowExec(gr, arg[0].c_str(),k-1,a);
 		if(n)		{	delete []a;	return n-1;	}
 		if(skip())	{	delete []a;	return 0;	}
@@ -598,11 +640,29 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 		{
 			if(k==3)
 			{
+				DeleteVar(arg[1].c_str());	// force to delete variable with the same name
 				mglNum *v=AddNum(arg[1].c_str());
-				v->d = mglFormulaCalc(arg[2],this, DataList).a[0];
+				if(arg[2][0]=='!')	// complex number is added
+				{	HADT dd = mglFormulaCalcC(arg[2].substr(1),this, DataList);
+					v->d=NAN;	v->c = dd->a[0];	delete dd;	}
+				else
+				{	HMDT dd = mglFormulaCalc(arg[2],this, DataList);
+					v->c = v->d = dd->a[0];	delete dd;	}
 			}
 			delete []a;	return k==3?0:1;
 		}
+		if(!arg[0].compare(L"rkstep"))
+		{
+			int res=1;
+			if(k>2 && a[0].type==1 && a[1].type==1)
+			{
+				std::wstring a1 = arg[1], a2=arg[2];	res = 0;
+				if(a1[0]=='\'')	a1 = a1.substr(1,a1.length()-2);
+				if(a2[0]=='\'')	a2 = a2.substr(1,a2.length()-2);
+				mgl_rk_step_w(this, a1.c_str(), a2.c_str(), (k>=3 && a[2].type==2)?a[2].v:1);
+			}
+			delete []a;	return res;
+		}
 		if(!arg[0].compare(L"call"))
 		{
 			n = 1;
@@ -836,7 +896,7 @@ void mglParser::Execute(mglGraph *gr, FILE *fp, bool print)
 	if(gr==0 || fp==0)	return;
 	std::wstring str;
 	wchar_t ch;
-	while((ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
+	while(!feof(fp) && size_t(ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
 //	while(!feof(fp))	str.push_back(fgetwc(fp));
 	Execute(gr,str.c_str());
 	if(print)	printf("%s\n",gr->Message());
@@ -859,13 +919,14 @@ void mglParser::Execute(mglGraph *gr, int n, const wchar_t **text)
 		else if(r==2)	snprintf(buf,64,"\nWrong command in line %ld\n", i+1);
 		else if(r==3)	snprintf(buf,64,"\nString too long in line %ld\n", i+1);
 		else if(r==4)	snprintf(buf,64,"\nUnbalanced ' in line %ld\n", i+1);
+		else if(r==5)	snprintf(buf,64,"\nChange temporary data in line %ld\n", i+1);
 		else if(gr->GetWarn()>0)	snprintf(buf,64," in line %ld\n", i+1);
 		else *buf=0;
 		buf[63] = 0;
 		if(*buf)	gr->SetWarn(-2,buf);
 		if(r>0 && r<5)	res=r;
 	}
-	int code[]={mglScrArg,	mglScrCmd,	mglScrLong,	mglScrStr};
+	int code[]={mglScrArg,	mglScrCmd,	mglScrLong,	mglScrStr, mglScrTemp};
 	if(res>0)	gr->SetWarn(code[res-1],"MGL Parser");
 }
 //-----------------------------------------------------------------------------
@@ -933,11 +994,11 @@ HMPR MGL_EXPORT mgl_create_parser()		{	return new mglParser;	}
 void MGL_EXPORT mgl_delete_parser(HMPR p)	{	delete p;	}
 void MGL_EXPORT mgl_parser_add_param(HMPR p, int id, const char *str)			{	p->AddParam(id,str);	}
 void MGL_EXPORT mgl_parser_add_paramw(HMPR p, int id, const wchar_t *str)		{	p->AddParam(id,str);	}
-HMDT MGL_EXPORT mgl_parser_add_var(HMPR p, const char *name)	{	return p->AddVar(name);	}
-MGL_EXPORT_PURE mglDataA *mgl_parser_find_var(HMPR p, const char *name)	{	return p->FindVar(name);}
+MGL_EXPORT mglDataA *mgl_parser_add_var(HMPR p, const char *name)	{	return p->AddVar(name);	}
+MGL_EXPORT mglDataA *mgl_parser_find_var(HMPR p, const char *name)	{	return p->FindVar(name);}
 void MGL_EXPORT mgl_parser_del_var(HMPR p, const char *name)	{	p->DeleteVar(name);	}
-HMDT MGL_EXPORT mgl_parser_add_varw(HMPR p, const wchar_t *name)	{	return p->AddVar(name);	}
-MGL_EXPORT_PURE mglDataA *mgl_parser_find_varw(HMPR p, const wchar_t *name)	{	return p->FindVar(name);}
+MGL_EXPORT mglDataA *mgl_parser_add_varw(HMPR p, const wchar_t *name)	{	return p->AddVar(name);	}
+MGL_EXPORT mglDataA *mgl_parser_find_varw(HMPR p, const wchar_t *name)	{	return p->FindVar(name);}
 void MGL_EXPORT mgl_parser_del_varw(HMPR p, const wchar_t *name)	{	p->DeleteVar(name);	}
 int MGL_EXPORT mgl_parse_line(HMGL gr, HMPR p, const char *str, int pos)
 {	return p->Parse(gr, str, pos);	}
@@ -964,9 +1025,9 @@ void MGL_EXPORT mgl_parser_add_param_(uintptr_t* p, int *id, const char *str, in
 /*===!!! NOTE !!! You must not delete obtained data arrays !!!===============*/
 uintptr_t MGL_EXPORT mgl_parser_add_var_(uintptr_t* p, const char *name, int l)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
-	mglData *v=_PR_->AddVar(s);	delete []s;	return uintptr_t(v);	}
+	mglDataA *v=_PR_->AddVar(s);	delete []s;	return uintptr_t(v);	}
 /*===!!! NOTE !!! You must not delete obtained data arrays !!!===============*/
-uintptr_t MGL_EXPORT_PURE mgl_parser_find_var_(uintptr_t* p, const char *name, int l)
+uintptr_t MGL_EXPORT mgl_parser_find_var_(uintptr_t* p, const char *name, int l)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
 	mglDataA *v=_PR_->FindVar(s);	delete []s;	return uintptr_t(v);	}
 void MGL_EXPORT mgl_parser_del_var_(uintptr_t* p, const char *name, int l)
@@ -989,52 +1050,56 @@ long MGL_EXPORT mgl_use_parser(HMPR pr, int inc)
 long MGL_EXPORT mgl_use_parser_(uintptr_t *p, int *inc)
 {	_PR_->InUse+=*inc;	return _PR_->InUse;	}
 //---------------------------------------------------------------------------
-MGL_EXPORT_PURE mglDataA *mgl_parser_get_var(HMPR p, unsigned long id)
+MGL_EXPORT mglDataA *mgl_parser_get_var(HMPR p, unsigned long id)
 {	return id<p->DataList.size()?p->DataList[id]:0;	}
-uintptr_t MGL_EXPORT_PURE mgl_parser_get_var_(uintptr_t* p, unsigned long *id)
+uintptr_t MGL_EXPORT mgl_parser_get_var_(uintptr_t* p, unsigned long *id)
 {	return uintptr_t(mgl_parser_get_var(_PR_,*id));	}
-long MGL_EXPORT_PURE mgl_parser_num_var(HMPR p)
+long MGL_EXPORT mgl_parser_num_var(HMPR p)
 {	return p->DataList.size();	}
-long MGL_EXPORT_PURE mgl_parser_num_var_(uintptr_t* p)
+long MGL_EXPORT mgl_parser_num_var_(uintptr_t* p)
 {	return mgl_parser_num_var(_PR_);	}
 //---------------------------------------------------------------------------
-int MGL_EXPORT_PURE mgl_parser_cmd_type(HMPR pr, const char *name)
+int MGL_EXPORT mgl_parser_cmd_type(HMPR pr, const char *name)
 {
 	const mglCommand *cmd = pr->FindCommand(name);
 	return cmd ? cmd->type + 1 : 0;
 }
-int MGL_EXPORT_PURE mgl_parser_cmd_type_(uintptr_t* p, const char *str, int l)
+int MGL_EXPORT mgl_parser_cmd_type_(uintptr_t* p, const char *str, int l)
 {	char *s=new char[l+1];	memcpy(s,str,l);	s[l]=0;
 	l = mgl_parser_cmd_type(_PR_, s);	delete []s;	return l;	}
 //---------------------------------------------------------------------------
-MGL_EXPORT_PURE const char *mgl_parser_cmd_desc(HMPR pr, const char *name)
+MGL_EXPORT const char *mgl_parser_cmd_desc(HMPR pr, const char *name)
 {
 	const mglCommand *cmd = pr->FindCommand(name);
 	return cmd ? cmd->desc : 0;
 }
-MGL_EXPORT_PURE const char *mgl_parser_cmd_frmt(HMPR pr, const char *name)
+MGL_EXPORT const char *mgl_parser_cmd_frmt(HMPR pr, const char *name)
 {
 	const mglCommand *cmd = pr->FindCommand(name);
 	return cmd ? cmd->form : 0;
 }
 //---------------------------------------------------------------------------
-MGL_EXPORT_PURE const char *mgl_parser_cmd_name(HMPR pr, long id)
+MGL_EXPORT const char *mgl_parser_cmd_name(HMPR pr, long id)
 {	return (id<mgl_parser_cmd_num(pr) && id>=0) ? pr->Cmd[id].name:"";	}
-long MGL_EXPORT_PURE mgl_parser_cmd_num(HMPR pr)
+long MGL_EXPORT mgl_parser_cmd_num(HMPR pr)
 {	register long i=0;	while(pr->Cmd[i].name[0])	i++; 	return i;	}
 //---------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_parser_calc(HMPR pr, const char *formula)
-{
-	HMDT d=0;
-	MGL_TO_WCS(formula,d = mgl_parser_calcw(pr,wcs));
-	return d;
-}
+{	HMDT d=0;	MGL_TO_WCS(formula,d = mgl_parser_calcw(pr,wcs));	return d;	}
 HMDT MGL_EXPORT mgl_parser_calcw(HMPR pr, const wchar_t *formula)
-{	mglData *d = new mglData(mglFormulaCalc(formula,pr, pr->DataList)); 	return d;	}
+{	return mglFormulaCalc(formula,pr, pr->DataList);	}
 uintptr_t MGL_EXPORT mgl_parser_calc_(uintptr_t *p, const char *str,int l)
 {	char *s=new char[l+1];	memcpy(s,str,l);	s[l]=0;
 	uintptr_t d = (uintptr_t)mgl_parser_calc(_PR_, s);	delete []s;	return d;	}
 //---------------------------------------------------------------------------
+HADT MGL_EXPORT mgl_parser_calc_complex(HMPR pr, const char *formula)
+{	HADT d=0;	MGL_TO_WCS(formula,d = mgl_parser_calc_complexw(pr,wcs));	return d;	}
+HADT MGL_EXPORT mgl_parser_calc_complexw(HMPR pr, const wchar_t *formula)
+{	return mglFormulaCalcC(formula,pr, pr->DataList);	}
+uintptr_t MGL_EXPORT mgl_parser_calc_complex_(uintptr_t *p, const char *str,int l)
+{	char *s=new char[l+1];	memcpy(s,str,l);	s[l]=0;
+	uintptr_t d = (uintptr_t)mgl_parser_calc_complex(_PR_, s);	delete []s;	return d;	}
+//---------------------------------------------------------------------------
 void MGL_EXPORT mgl_parser_del_all(HMPR p)	{	p->DeleteAll();	}
 void MGL_EXPORT mgl_parser_del_all_(uintptr_t *p)	{	_PR_->DeleteAll();	}
 //---------------------------------------------------------------------------
@@ -1067,3 +1132,178 @@ void MGL_EXPORT mgl_parser_load_(uintptr_t *p, const char *dll_name,int l)
 {	char *s=new char[l+1];	memcpy(s,dll_name,l);	s[l]=0;
 	mgl_parser_load(_PR_, s);	delete []s;	}
 //---------------------------------------------------------------------------
+struct mglRKdat
+{
+	mglDataA *v;
+	std::wstring e;
+	bool cmplx;
+	mglDataC cin,c1,c2,c3,c4, *cc;
+	mglData  din,d1,d2,d3,d4, *dd;
+	mglRKdat(mglDataA *var, std::wstring &eq):v(var), e(eq)
+	{	cmplx = dynamic_cast<mglDataC*>(var);	cc=0;	dd=0;	}
+	void allocate()
+	{
+		if(cmplx)
+		{	cc = dynamic_cast<mglDataC*>(v);	cin.Set(v);	}
+		else
+		{	dd = dynamic_cast<mglData*>(v);		din.Set(v);	}
+	}
+};
+void MGL_EXPORT mgl_rk_step_w(HMPR pr, const wchar_t *Eqs, const wchar_t *Vars, mreal dt)
+{
+	const std::wstring eqs(Eqs);
+	const std::wstring vars(Vars);
+	std::vector<mglRKdat> rkv;
+	size_t iv=0,jv=0,ie=0,je=0;
+	while(1)
+	{
+		iv = vars.find(';',jv);	ie = eqs.find(';',je);
+		mglDataA *vv=mgl_parser_find_varw(pr,vars.substr(jv,iv-jv).c_str());
+		std::wstring eq = eqs.substr(je,ie-je).c_str();
+		if(vv)	rkv.push_back(mglRKdat(vv, eq ));
+		jv = iv+1;	je = ie+1;
+		if(iv==std::wstring::npos || ie==std::wstring::npos)	break;
+	}
+	for(size_t i=0;i<rkv.size();i++)	rkv[i].allocate();
+	mreal hh = dt/2;
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cmplx)	rk.c1.Move(mglFormulaCalcC(rk.e, pr, pr->DataList));
+		else	rk.d1.Move(mglFormulaCalc(rk.e, pr, pr->DataList));
+	}
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cc)
+		{
+			long n = rk.cc->GetNN();	dual a = hh*rk.c1.a[0];
+			if(rk.c1.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + hh*rk.c1.a[j];
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + a;
+		}
+		if(rk.dd)
+		{
+			long n = rk.dd->GetNN();	mreal a = hh*rk.d1.a[0];
+			if(rk.d1.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + hh*rk.d1.a[j];
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + a;
+		}
+	}
+
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cmplx)	rk.c2.Move(mglFormulaCalcC(rk.e, pr, pr->DataList));
+		else	rk.d2.Move(mglFormulaCalc(rk.e, pr, pr->DataList));
+	}
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cc)
+		{
+			long n = rk.cc->GetNN();	dual a = hh*rk.c2.a[0];
+			if(rk.c2.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + hh*rk.c2.a[j];
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + a;
+		}
+		if(rk.dd)
+		{
+			long n = rk.dd->GetNN();	mreal a = hh*rk.d2.a[0];
+			if(rk.d2.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + hh*rk.d2.a[j];
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + a;
+		}
+	}
+
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cmplx)	rk.c3.Move(mglFormulaCalcC(rk.e, pr, pr->DataList));
+		else	rk.d3.Move(mglFormulaCalc(rk.e, pr, pr->DataList));
+	}
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cc)
+		{
+			long n = rk.cc->GetNN();	dual a = dt*rk.c3.a[0];
+			if(rk.c3.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + dt*rk.c3.a[j];
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + a;
+		}
+		if(rk.dd)
+		{
+			long n = rk.dd->GetNN();	mreal a = dt*rk.d3.a[0];
+			if(rk.d3.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + dt*rk.d3.a[j];
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + a;
+		}
+	}
+
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cmplx)	rk.c4.Move(mglFormulaCalcC(rk.e, pr, pr->DataList));
+		else	rk.d4.Move(mglFormulaCalc(rk.e, pr, pr->DataList));
+	}
+	for(size_t i=0;i<rkv.size();i++)
+	{
+		mglRKdat &rk = rkv[i];
+		if(rk.cc)
+		{
+			long n = rk.cc->GetNN();
+			dual a = (rk.c1.a[0]+rk.c2.a[0]+mreal(2)*(rk.c3.a[0]+rk.c4.a[0]))*(dt/6);
+			if(rk.c1.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + (rk.c1.a[j]+rk.c2.a[j]+mreal(2)*(rk.c3.a[j]+rk.c4.a[j]))*(dt/6);
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.cc->a[j] = rk.cin.a[j] + a;
+		}
+		if(rk.dd)
+		{
+			long n = rk.dd->GetNN();
+			mreal a = (rk.d1.a[0]+rk.d2.a[0]+2*(rk.d3.a[0]+rk.d4.a[0]))*(dt/6);
+			if(rk.d1.GetNN()==n)
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + (rk.d1.a[j]+rk.d2.a[j]+2*(rk.d3.a[j]+rk.d4.a[j]))*(dt/6);
+			else
+#pragma omp parallel for
+				for(long j=0;j<n;j++)	rk.dd->a[j] = rk.din.a[j] + a;
+		}
+	}
+}
+void MGL_EXPORT mgl_rk_step(HMPR pr, const char *Eqs, const char *Vars, mreal dt)
+{
+	if(Eqs && *Eqs && Vars && *Vars)
+	{
+		size_t s=mbstowcs(0,Eqs,0), w=mbstowcs(0,Vars,0);
+		wchar_t *eqs=new wchar_t[s+1];	mbstowcs(eqs,Eqs ,s);	eqs[s]=0;
+		wchar_t *wcs=new wchar_t[s+1];	mbstowcs(wcs,Vars,s);	wcs[w]=0;
+		mgl_rk_step_w(pr,eqs,wcs,dt);	delete []wcs;	delete []eqs;
+	}
+}
+void MGL_EXPORT mgl_rk_step_(uintptr_t *p, const char *eqs, const char *vars, double *dt,int l,int m)
+{	char *e=new char[l+1];	memcpy(e,eqs,l);	e[l]=0;
+	char *s=new char[m+1];	memcpy(s,vars,m);	s[m]=0;
+	mgl_rk_step(_PR_,e,s,*dt);	delete []e;	delete []s;	}
+//---------------------------------------------------------------------------
