@@ -57,7 +57,7 @@ void MGL_EXPORT mgl_cloud_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const cha
 	n /= tx;	m /= ty;	l /= tz;
 	long *pos=new long[n*m*l];
 	gr->Reserve(n*m*l);
-	mglPoint q=mglPoint(NAN);
+	mglPoint q(NAN);
 	for(long k=0;k<l;k++)
 	{
 		if(gr->NeedStop())	break;
@@ -182,7 +182,7 @@ void MGL_EXPORT mgl_surf3_plot(HMGL gr, long n,long m,long *kx1,long *kx2,long *
 		if(ni<3)	continue;
 
 		for(jj=0;jj<ni;jj++)
-		{	pp[jj]=kk[id[jj]];	pd[jj]=long(pp[jj].c+0.5);	}
+		{	pp[jj]=kk[id[jj]];	pd[jj]=mgl_int(pp[jj].c);	}
 		// remove points which is too close to first one
 		for(jj=1;jj<ni;)
 		{
@@ -235,90 +235,98 @@ void MGL_EXPORT mgl_surf3_plot(HMGL gr, long n,long m,long *kx1,long *kx2,long *
 	}
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_surf3_xyz_val(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+void MGL_NO_EXPORT mgl_surf3ca_gen(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT c, HCDT b, const char *sch)
 {
-	long i,j,k,i1,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	long *kx1,*kx2,*ky1,*ky2,*kz;
-	bool nboth = !mgl_isboth(x,y,z,a);
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	bool nboth = mgl_isnboth(x,y,z,a);
 	int wire = mglchr(sch,'#')?1:0;
 	if(mglchr(sch,'.'))	wire = 2;
-	mreal d;
-	if(mgl_check_dim3(gr,!nboth,x,y,z,a,0,"Surf3"))	return;
-
-	gr->SaveState(opt);
-	static int cgid=1;	gr->StartGroup("Surf3",cgid++);
-
 	bool inv = (mglchr(sch,'-'));
-	long ss = gr->AddTexture(sch), pos;
+	long ss = gr->AddTexture(sch);
 
-	kx1 = new long[n*m];	kx2 = new long[n*m];
-	ky1 = new long[n*m];	ky2 = new long[n*m];
-	kz  = new long[n*m];
-	mreal c=gr->GetC(ss,val);
+	long *kx1 = new long[n*m],	*kx2 = new long[n*m];
+	long *ky1 = new long[n*m],	*ky2 = new long[n*m];
+	long *kz  = new long[n*m];
 	std::vector<mglPoint> kk;
-//	kk.reserve(n*m*l);
+	kk.reserve(n*m*l);
 
-	mglPoint p,q,u, p0;
-	mreal a0;
-	for(k=0;k<l;k++)
+	for(long k=0;k<l;k++)
 	{
 		if(gr->NeedStop())	break;
 		memcpy(kx1,kx2,n*m*sizeof(long));	memset(kx2,-1,n*m*sizeof(long));
 		memcpy(ky1,ky2,n*m*sizeof(long));	memset(ky2,-1,n*m*sizeof(long));
 		memset(kz ,-1,n*m*sizeof(long));
 		gr->Reserve(n*m);	gr->Reserve(n*m);
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+		size_t kk1 = kk.size();
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			i1 = i+n*j;		a0 = a->v(i,j,k);
-			p0 = nboth? mglPoint(x->v(i), y->v(j), z->v(k)) : mglPoint(x->v(i,j,k), y->v(i,j,k), z->v(i,j,k));
+			register long i1 = i+n*j;
+			mreal a0 = a->v(i,j,k);
 			if(i<n-1)
 			{
-				d = mgl_d(val,a0,a->v(i+1,j,k));
+				mreal d = mgl_d(val,a0,a->v(i+1,j,k));
 				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x*(1-d)+x->v(i+1)*d, p0.y, p0.z);
-					else	p = mglPoint(p0.x*(1-d)+x->v(i+1,j,k)*d, p0.y*(1-d)+y->v(i+1,j,k)*d, p0.z*(1-d)+z->v(i+1,j,k)*d);
-					u = mglPoint(i+d,j,k);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q);	u.c=pos;
-					if(pos<0)	continue;
-					kx2[i1] = kk.size();	kk.push_back(u);
-				}
+				{	kx2[i1] = kk.size();	kk.push_back(mglPoint(i+d,j,k));	}
 			}
 			if(j<m-1)
 			{
-				d = mgl_d(val,a0,a->v(i,j+1,k));
+				mreal d = mgl_d(val,a0,a->v(i,j+1,k));
 				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x, p0.y*(1-d)+y->v(j+1)*d, p0.z);
-					else	p = mglPoint(p0.x*(1-d)+x->v(i,j+1,k)*d, p0.y*(1-d)+y->v(i,j+1,k)*d, p0.z*(1-d)+z->v(i,j+1,k)*d);
-					u = mglPoint(i,j+d,k);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q);	u.c=pos;
-					if(pos<0)	continue;
-					ky2[i1] = kk.size();	kk.push_back(u);
-				}
+				{	ky2[i1] = kk.size();	kk.push_back(mglPoint(i,j+d,k));	}
 			}
 			if(k>0)
-			{
-				d = mgl_d(val,a->v(i,j,k-1),a0);
+			{	
+				mreal d = mgl_d(val,a->v(i,j,k-1),a0);
 				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x, p0.y, z->v(k-1)*(1-d)+p0.z*d);
-					else	p = mglPoint(x->v(i,j,k-1)*(1-d)+p0.x*d, y->v(i,j,k-1)*(1-d)+p0.y*d, z->v(i,j,k-1)*(1-d)+p0.z*d);
-					u = mglPoint(i,j,k+d-1);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q);	u.c=pos;
-					if(pos<0)	continue;
-					kz[i1] = kk.size();	kk.push_back(u);
-				}
+				{	kz[i1] = kk.size();		kk.push_back(mglPoint(i,j,k+d-1));	}
 			}
 		}
+		mreal cv=gr->GetC(ss,val);
+		if(b && c)	for(size_t i=kk1;i<kk.size();i++)
+		{
+			mglPoint &u = kk[i];
+			u.c = gr->AddPnt(nboth ? mglPoint(mgl_data_linear(x,u.x,0,0),mgl_data_linear(y,u.y,0,0),mgl_data_linear(z,u.z,0,0)) : 
+							mglPoint(mgl_data_linear(x,u.x,u.y,u.z),mgl_data_linear(y,u.x,u.y,u.z),mgl_data_linear(z,u.x,u.y,u.z)),
+					gr->GetC(ss,mgl_data_linear(c,u.x,u.y,u.z)),
+					mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l),
+					gr->GetA(mgl_data_linear(b,u.x,u.y,u.z)));
+		}
+		else if(c)	for(size_t i=kk1;i<kk.size();i++)
+		{
+			mglPoint &u = kk[i];
+			u.c = gr->AddPnt(nboth ? mglPoint(mgl_data_linear(x,u.x,0,0),mgl_data_linear(y,u.y,0,0),mgl_data_linear(z,u.z,0,0)) : 
+							mglPoint(mgl_data_linear(x,u.x,u.y,u.z),mgl_data_linear(y,u.x,u.y,u.z),mgl_data_linear(z,u.x,u.y,u.z)),
+						gr->GetC(ss,mgl_data_linear(c,u.x,u.y,u.z)),
+						mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l));
+		}
+		else if(b)	for(size_t i=kk1;i<kk.size();i++)
+		{
+			mglPoint &u = kk[i];
+			u.c = gr->AddPnt(nboth ? mglPoint(mgl_data_linear(x,u.x,0,0),mgl_data_linear(y,u.y,0,0),mgl_data_linear(z,u.z,0,0)) : 
+							mglPoint(mgl_data_linear(x,u.x,u.y,u.z),mgl_data_linear(y,u.x,u.y,u.z),mgl_data_linear(z,u.x,u.y,u.z)),
+					cv, mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l),
+					gr->GetA(mgl_data_linear(b,u.x,u.y,u.z)));
+		}
+		else	for(size_t i=kk1;i<kk.size();i++)
+		{
+			mglPoint &u = kk[i];
+			u.c = gr->AddPnt(nboth ? mglPoint(mgl_data_linear(x,u.x,0,0),mgl_data_linear(y,u.y,0,0),mgl_data_linear(z,u.z,0,0)) : 
+							mglPoint(mgl_data_linear(x,u.x,u.y,u.z),mgl_data_linear(y,u.x,u.y,u.z),mgl_data_linear(z,u.x,u.y,u.z)),
+						cv, mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l));
+		}
+		
 		if(k>0)	mgl_surf3_plot(gr,n,m,kx1,kx2,ky1,ky2,kz,kk,wire);
 	}
-	gr->EndGroup();
 	delete []kx1;	delete []kx2;	delete []ky1;
-	delete []ky2;	delete []kz;
+	delete []ky2;	delete []kz;	gr->EndGroup();
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3_xyz_val(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+{
+	if(mgl_check_dim3(gr,mgl_isboth(x,y,z,a),x,y,z,a,0,"Surf3"))	return;
+	gr->SaveState(opt);
+	static int cgid=1;	gr->StartGroup("Surf3",cgid++);
+	mgl_surf3ca_gen(gr, val, x, y, z, a, 0, 0, sch);
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surf3_val(HMGL gr, double val, HCDT a, const char *sch, const char *opt)
@@ -378,92 +386,10 @@ void MGL_EXPORT mgl_surf3_(uintptr_t *gr, uintptr_t *a, const char *sch, const c
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surf3a_xyz_val(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *sch, const char *opt)
 {
-	long i,j,k,i1,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	long *kx1,*kx2,*ky1,*ky2,*kz;
-	bool nboth = mgl_isnboth(x,y,z,a);
-	int wire = mglchr(sch,'#')?1:0;
-	if(mglchr(sch,'.'))	wire = 2;
-	mreal d;
-	if(mgl_check_dim3(gr,!nboth,x,y,z,a,b,"Surf3A"))	return;
-
+	if(mgl_check_dim3(gr,!mgl_isnboth(x,y,z,a),x,y,z,a,b,"Surf3A"))	return;
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Surf3A",cgid++);
-
-	bool inv = (mglchr(sch,'-'));
-	long ss = gr->AddTexture(sch), pos;
-
-	kx1 = new long[n*m];	kx2 = new long[n*m];
-	ky1 = new long[n*m];	ky2 = new long[n*m];
-	kz  = new long[n*m];
-	mreal c=gr->GetC(ss,val),aa;
-	std::vector<mglPoint> kk;
-	kk.reserve(n*m*l);
-
-	mglPoint p,q,u, p0;
-	mreal a0,b0;
-	for(k=0;k<l;k++)
-	{
-		if(gr->NeedStop())	break;
-		memcpy(kx1,kx2,n*m*sizeof(long));	memset(kx2,-1,n*m*sizeof(long));
-		memcpy(ky1,ky2,n*m*sizeof(long));	memset(ky2,-1,n*m*sizeof(long));
-		memset(kz ,-1,n*m*sizeof(long));
-		gr->Reserve(n*m);	gr->Reserve(n*m);
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
-		{
-			i1 = i+n*j;
-			a0 = a->v(i,j,k);	b0 = b->v(i,j,k);
-			p0 = nboth? mglPoint(x->v(i), y->v(j), z->v(k)) : mglPoint(x->v(i,j,k), y->v(i,j,k), z->v(i,j,k));
-			if(i<n-1)
-			{
-				d = mgl_d(val,a0,a->v(i+1,j,k));
-				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x*(1-d)+x->v(i+1)*d, p0.y, p0.z);
-					else	p = mglPoint(p0.x*(1-d)+x->v(i+1,j,k)*d, p0.y*(1-d)+y->v(i+1,j,k)*d, p0.z*(1-d)+z->v(i+1,j,k)*d);
-					aa = gr->GetA(b0*(1-d)+b->v(i+1,j,k)*d);
-					u = mglPoint(i+d,j,k);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q,aa);	u.c=pos;
-					if(pos<0)	continue;
-					kx2[i1] = kk.size();	kk.push_back(u);
-				}
-			}
-			if(j<m-1)
-			{
-				d = mgl_d(val,a0,a->v(i,j+1,k));
-				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x, p0.y*(1-d)+y->v(j+1)*d, p0.z);
-					else	p = mglPoint(p0.x*(1-d)+x->v(i,j+1,k)*d, p0.y*(1-d)+y->v(i,j+1,k)*d, p0.z*(1-d)+z->v(i,j+1,k)*d);
-					aa = gr->GetA(b0*(1-d)+b->v(i,j+1,k)*d);
-					u = mglPoint(i,j+d,k);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q,aa);	u.c=pos;
-					if(pos<0)	continue;
-					ky2[i1] = kk.size();	kk.push_back(u);
-				}
-			}
-			if(k>0)
-			{
-				d = mgl_d(val,a->v(i,j,k-1),a0);
-				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x, p0.y, z->v(k-1)*(1-d)+p0.z*d);
-					else	p = mglPoint(x->v(i,j,k-1)*(1-d)+p0.x*d, y->v(i,j,k-1)*(1-d)+p0.y*d, z->v(i,j,k-1)*(1-d)+p0.z*d);
-					aa = gr->GetA(b->v(i,j,k-1)*(1-d)+b0*d);
-					u = mglPoint(i,j,k+d-1);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q,aa);	u.c=pos;
-					if(pos<0)	continue;
-					kz[i1] = kk.size();	kk.push_back(u);
-				}
-			}
-		}
-		if(k>0)	mgl_surf3_plot(gr,n,m,kx1,kx2,ky1,ky2,kz,kk,wire);
-	}
-	gr->EndGroup();
-	delete []kx1;	delete []kx2;	delete []ky1;
-	delete []ky2;	delete []kz;
+	mgl_surf3ca_gen(gr, val, x, y, z, a, 0, b, sch);
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surf3a_val(HMGL gr, double val, HCDT a, HCDT b, const char *sch, const char *opt)
@@ -535,94 +461,12 @@ void MGL_EXPORT mgl_surf3a_(uintptr_t *gr, uintptr_t *a, uintptr_t *b, const cha
 //	Surf3C series
 //
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_surf3c_xyz_val(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *sch, const char *opt)
+void MGL_EXPORT mgl_surf3c_xyz_val(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT c, const char *sch, const char *opt)
 {
-	long i,j,k,i1,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	long *kx1,*kx2,*ky1,*ky2,*kz;
-	bool nboth = mgl_isnboth(x,y,z,a);
-	int wire = mglchr(sch,'#')?1:0;
-	if(mglchr(sch,'.'))	wire = 2;
-	mreal d;
-	if(mgl_check_dim3(gr,!nboth,x,y,z,a,b,"Surf3C"))	return;
-
+	if(mgl_check_dim3(gr,!mgl_isnboth(x,y,z,a),x,y,z,a,c,"Surf3C"))	return;
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Surf3C",cgid++);
-
-	bool inv = (mglchr(sch,'-'));
-	long ss = gr->AddTexture(sch), pos;
-
-	kx1 = new long[n*m];	kx2 = new long[n*m];
-	ky1 = new long[n*m];	ky2 = new long[n*m];
-	kz  = new long[n*m];
-	mreal c;
-	std::vector<mglPoint> kk;
-	kk.reserve(n*m*l);
-
-	mglPoint p,q,u, p0;
-	mreal a0,b0;
-	for(k=0;k<l;k++)
-	{
-		if(gr->NeedStop())	break;
-		memcpy(kx1,kx2,n*m*sizeof(long));	memset(kx2,-1,n*m*sizeof(long));
-		memcpy(ky1,ky2,n*m*sizeof(long));	memset(ky2,-1,n*m*sizeof(long));
-		memset(kz ,-1,n*m*sizeof(long));
-		gr->Reserve(n*m);	gr->Reserve(n*m);
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
-		{
-			i1 = i+n*j;
-			a0 = a->v(i,j,k);	b0 = b->v(i,j,k);
-			p0 = nboth? mglPoint(x->v(i), y->v(j), z->v(k)) : mglPoint(x->v(i,j,k), y->v(i,j,k), z->v(i,j,k));
-			if(i<n-1)
-			{
-				d = mgl_d(val,a0,a->v(i+1,j,k));
-				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x*(1-d)+x->v(i+1)*d, p0.y, p0.z);
-					else	p = mglPoint(p0.x*(1-d)+x->v(i+1,j,k)*d, p0.y*(1-d)+y->v(i+1,j,k)*d, p0.z*(1-d)+z->v(i+1,j,k)*d);
-					c = gr->GetC(ss,b0*(1-d)+b->v(i+1,j,k)*d);
-					u = mglPoint(i+d,j,k);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q);	u.c=pos;
-					if(pos<0)	continue;
-					kx2[i1] = kk.size();	kk.push_back(u);
-				}
-			}
-			if(j<m-1)
-			{
-				d = mgl_d(val,a0,a->v(i,j+1,k));
-				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x, p0.y*(1-d)+y->v(j+1)*d, p0.z);
-					else	p = mglPoint(p0.x*(1-d)+x->v(i,j+1,k)*d, p0.y*(1-d)+y->v(i,j+1,k)*d, p0.z*(1-d)+z->v(i,j+1,k)*d);
-					c = gr->GetC(ss,b0*(1-d)+b->v(i,j+1,k)*d);
-					u = mglPoint(i,j+d,k);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q);	u.c=pos;
-					if(pos<0)	continue;
-					ky2[i1] = kk.size();	kk.push_back(u);
-				}
-			}
-			if(k>0)
-			{
-				d = mgl_d(val,a->v(i,j,k-1),a0);
-				if(d>=0 && d<1)
-				{
-					if(nboth)	p = mglPoint(p0.x, p0.y, z->v(k-1)*(1-d)+p0.z*d);
-					else	p = mglPoint(x->v(i,j,k-1)*(1-d)+p0.x*d, y->v(i,j,k-1)*(1-d)+p0.y*d, z->v(i,j,k-1)*(1-d)+p0.z*d);
-					c = gr->GetC(ss,b->v(i,j,k-1)*(1-d)+b0*d);
-					u = mglPoint(i,j,k+d-1);
-					q = mgl_find_norm(nboth, x,y,z,a, u, inv,n,m,l);
-					pos = gr->AddPnt(p,c,q);	u.c=pos;
-					if(pos<0)	continue;
-					kz[i1] = kk.size();	kk.push_back(u);
-				}
-			}
-		}
-		if(k>0)	mgl_surf3_plot(gr,n,m,kx1,kx2,ky1,ky2,kz,kk,wire);
-	}
-	gr->EndGroup();
-	delete []kx1;	delete []kx2;	delete []ky1;
-	delete []ky2;	delete []kz;
+	mgl_surf3ca_gen(gr, val, x, y, z, a, c, 0, sch);
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surf3c_val(HMGL gr, double val, HCDT a, HCDT b, const char *sch, const char *opt)
@@ -680,6 +524,72 @@ void MGL_EXPORT mgl_surf3c_(uintptr_t *gr, uintptr_t *a, uintptr_t *b, const cha
 	mgl_surf3c(_GR_, _DA_(a), _DA_(b), s, o);	delete []o;	delete []s;	}
 //-----------------------------------------------------------------------------
 //
+//	Surf3C series
+//
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_xyz_val(HMGL gr, double val, HCDT x, HCDT y, HCDT z, HCDT a, HCDT c, HCDT b, const char *sch, const char *opt)
+{
+	if(mgl_check_dim3(gr,!mgl_isnboth(x,y,z,a),x,y,z,a,c,"Surf3C") || mgl_check_dim3(gr,!mgl_isnboth(x,y,z,a),x,y,z,a,b,"Surf3C"))	return;
+	gr->SaveState(opt);
+	static int cgid=1;	gr->StartGroup("Surf3C",cgid++);
+	mgl_surf3ca_gen(gr, val, x, y, z, a, c, b, sch);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_val(HMGL gr, double val, HCDT a, HCDT c, HCDT b, const char *sch, const char *opt)
+{
+	gr->SaveState(opt);
+	mglDataV x(a->GetNx()), y(a->GetNy()),z(a->GetNz());
+	x.Fill(gr->Min.x,gr->Max.x);
+	y.Fill(gr->Min.y,gr->Max.y);
+	z.Fill(gr->Min.z,gr->Max.z);
+	mgl_surf3ca_xyz_val(gr,val,&x,&y,&z,a,c,b,sch,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, HCDT c, HCDT b, const char *sch, const char *opt)
+{
+	mreal r = gr->SaveState(opt);
+	long num = mgl_isnan(r)?3:long(r+0.5);
+	for(long i=0;i<num;i++)
+	{
+		mreal v = gr->Max.c + (gr->Min.c-gr->Max.c)*(i+1.)/(num+1);
+		mgl_surf3ca_xyz_val(gr,v,x,y,z,a,c,b,sch,0);
+	}
+	gr->LoadState();
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca(HMGL gr, HCDT a, HCDT c, HCDT b, const char *sch, const char *opt)
+{
+	gr->SaveState(opt);
+	mglDataV x(a->GetNx()), y(a->GetNy()),z(a->GetNz());
+	x.Fill(gr->Min.x,gr->Max.x);
+	y.Fill(gr->Min.y,gr->Max.y);
+	z.Fill(gr->Min.z,gr->Max.z);
+	mgl_surf3ca_xyz(gr,&x,&y,&z,a,c,b,sch,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_xyz_val_(uintptr_t *gr, mreal *Val, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_surf3ca_xyz_val(_GR_, *Val, _DA_(x), _DA_(y), _DA_(z), _DA_(a), _DA_(c), _DA_(b), s, o);
+	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_val_(uintptr_t *gr, mreal *Val, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	 mgl_surf3ca_val(_GR_, *Val, _DA_(a), _DA_(c), _DA_(b), s, o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_surf3ca_xyz(_GR_, _DA_(x), _DA_(y), _DA_(z), _DA_(a), _DA_(c), _DA_(b), s, o);
+	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_surf3ca_(uintptr_t *gr, uintptr_t *a, uintptr_t *c, uintptr_t *b, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_surf3ca(_GR_, _DA_(a), _DA_(c), _DA_(b), s, o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+//
 //	Beam series
 //
 //-----------------------------------------------------------------------------
