@@ -35,7 +35,7 @@ int main(int argc, char *argv[])
 
 	while(1)
 	{
-		ch = getopt(argc, argv, "1:2:3:4:5:6:7:8:9:hno:L:C:A:s:");
+		ch = getopt(argc, argv, "1:2:3:4:5:6:7:8:9:hno:L:C:A:s:S:q:");
 		if(ch>='1' && ch<='9')	p.AddParam(ch-'0', optarg);
 		else if(ch=='s')
 		{
@@ -44,12 +44,14 @@ int main(int argc, char *argv[])
 			if(fp)
 			{
 				wchar_t ch;
-				while((ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
+				while(!feof(fp) && size_t(ch=fgetwc(fp))!=WEOF)	str.push_back(ch);
 				fclose(fp);	str += L"\n";
 			}
 		}
 		else if(ch=='n')	none = true;
 		else if(ch=='L')	setlocale(LC_CTYPE, optarg);
+		else if(ch=='S')	mgl_set_size_scl(atof(optarg));
+		else if(ch=='q')	gr.SetQuality(atoi(optarg));
 		else if(ch=='A')
 		{
 			std::wstring str;
@@ -78,6 +80,8 @@ int main(int argc, char *argv[])
 				"\t-9 str       set str as argument $9 for script\n"
 				"\t-L loc       set locale to loc\n"
 				"\t-s opt       set MGL script for setting up the plot\n"
+				"\t-S val       set scaling factor for images\n"
+				"\t-q val       set quality for output (val=0...9)\n"
 				"\t-o name      set output file name\n"
 				"\t-n           no manual output (script should save results by itself)\n"
 				"\t-A val       add animation value val\n"
@@ -101,8 +105,7 @@ int main(int argc, char *argv[])
 	FILE *fp = *iname?fopen(iname,"r"):stdin;
 	if(!fp)	{	printf("No file for MGL script\n");	return 0;	}
 	wchar_t cw;
-	while((cw=fgetwc(fp))!=WEOF)	str.push_back(cw);
-//	while(!feof(fp))	str.push_back(fgetwc(fp));
+	while(!feof(fp) && size_t(cw=fgetwc(fp))!=WEOF)	str.push_back(cw);
 	if(*iname)	fclose(fp);
 
 	unsigned long n;
@@ -141,7 +144,7 @@ int main(int argc, char *argv[])
 	else
 	{
 		p.Execute(&gr,str.c_str());
-		printf("%s\n",gr.Message());
+		if(gr.Message()[0])	printf("%s\n",gr.Message());
 		if(!none)	gr.WriteFrame(oname);
 	}
 	if(!mglGlobalMess.empty())	printf("%s",mglGlobalMess.c_str());
